@@ -40,6 +40,7 @@
 #include <math.h>
 #include <stdlib.h>
 #include <memory.h>
+#include <assert.h>
 
 // Conversion-Constants
 
@@ -176,9 +177,9 @@ class C_RECORD
 {
 public:
   /** Number of waypoints */
-  word NTP;
+  unsigned NTP;
   /** Task-ID */
-  word TID;
+  unsigned TID;
   /** Time of declaration */
   struct tm TDECL;
   int hasdeclaration;
@@ -202,7 +203,6 @@ public:
   void
   print(int version, FILE *aus)
   {
-    word i;
     char is[8];
 
     if (hasdeclaration) {
@@ -250,7 +250,7 @@ public:
       STA.print(version, aus, "Start  "); // print Start-Point
 
       // print Turnpoints
-      for (i = 0; i < NTP; i++) {
+      for (unsigned i = 0; i < NTP; i++) {
         sprintf(is, "TP%02u   ", i + 1);
         TP[i].print(version, aus, is);
       }
@@ -353,7 +353,7 @@ struct IGCHEADER
    * Unused field will be prepared as HO fields.
    */
   void
-  output(int version, int oo_fillin)
+  output(int version, bool oo_fillin)
   {
     igc_filter(PLT);
     igc_filter(GTY);
@@ -418,7 +418,7 @@ struct IGCHEADER
     }
 
     // LCONV-VER-Erzeugung ab Version 4.16
-    if ((version >= 416) && (oo_fillin == 1))
+    if (version >= 416 && oo_fillin)
       fprintf(ausgabe, "LCONV-VER:%01d.%02d\n", version / 100, version % 100);
   }
 };
@@ -431,11 +431,11 @@ struct IGCHEADER
  * more or less inaccurate conversion from HDOP to fix accuracy.
  * doesn't have to be right. might be changed sometime.
  */
-static word
+static unsigned
 hdop2fxa(uint8_t hdop)
 {
   // return (float) hdop * 100.0/3);
-  return word((float(hdop) * 100.01 / 3));
+  return unsigned((float(hdop) * 100.01 / 3));
 }
 
 /** non-linear conversion of the ENL values */
@@ -510,10 +510,13 @@ Parameter
 
 const int actual_conv_version = 424;
 
-long
-convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
-    int oo_fillin, word *serno, long *sp)
+size_t
+convert_gcs(int igcfile_version, FILE *Ausgabedatei,
+            const uint8_t *const bin_puffer, size_t length,
+            bool oo_fillin)
 {
+  const uint8_t *const end = bin_puffer + length;
+
   IGCHEADER igcheader;
   C_RECORD task;
   struct
@@ -522,18 +525,7 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
     char time[10];
     char valid;
     long lat;
-    word latdeg;
-    word latmin;
     long lon;
-    word londeg;
-    word lonmin;
-    word press;
-    word gpalt;
-    long pressure_alt;
-    long gps_alt;
-    word fxa;
-    word hdop;
-    word enl;
   } igcfix;
 
   int l = 0;
@@ -549,8 +541,8 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
   tm realtime;
   uint8_t Haupttyp;
   uint8_t Untertyp;
-  uint8_t *p;
-  uint8_t *p2;
+  const uint8_t *p;
+  const uint8_t *p2;
   long pl;
   char PILOT[40];
   int tzh, tzm;
@@ -582,9 +574,15 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
   p = bin_puffer;
 
   do {
+    if (p >= end)
+      return 0;
+
     Haupttyp = p[0] & rectyp_msk;
     switch (Haupttyp) {
     case rectyp_tnd:
+      if (p + 8 > end)
+        return 0;
+
       // calculates backwards the time of the first fix
       time_relative += p[1];
       temptime = 65536L * p[2] + 256L * p[3] + p[4];
@@ -623,11 +621,18 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
        */
     case rectyp_pos:
     case rectyp_poc:
+      if (p + 2 >= end)
+        return 0;
+
       if (p[2] & 0x80) { // Endebedingung
         ende = 1;
         l = 0;
         break;
       }
+
+      if (p + 8 > end)
+        return 0;
+
       time_relative += p[2];
       igcfix.valid = ((p[0] & 0x10) >> 4) ? 'A' : 'V';
       if (Haupttyp == rectyp_pos) {
@@ -665,12 +670,15 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
       l = 1; // Fllzeichen
       break;
     case rectyp_end:
-      *sp = (p - bin_puffer) + 1;
+      /* sp = (p - bin_puffer) + 1 */
       l = 41;
       ende = 1;
       break;
     case rectyp_vrb:
     case rectyp_vrt:
+      if (p + 2 >= end)
+        return 0;
+
       l = p[1];
       switch (Haupttyp) {
       case rectyp_vrb:
@@ -684,23 +692,39 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
         p2 = p;
         break;
       }
+
+      if (p2 >= end)
+        return 0;
+
       Untertyp = (p2[0]);
       switch (Untertyp) {
 
       case FLDNTP:
+        if (p2 + 1 >= end)
+          return 0;
+
         task.NTP = p2[1];
         decl_time = time_relative;
         break;
       case FLDTID:
+        if (p2 + 2 >= end)
+          return 0;
+
         task.TID = 256 * p2[1] + p2[2];
         if (igcfile_version >= 422)
           decl_time = time_relative;
         break;
       case FLDFDT:
+        if (p2 + 1 + sizeof(task.FDT) > end)
+          return 0;
+
         //_fmemcpy(&task.FDT,&p2[1],sizeof task.FDT);
         memcpy(&task.FDT, &p2[1], sizeof task.FDT);
         break;
       case FLDTZN: // Reading timezone offset
+        if (p2 + 1 >= end)
+          return 0;
+
         if (p2[1] < 128)
           tzn = 15 * p2[1];
         else
@@ -708,51 +732,96 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
         break;
 
       case FLDTKF:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TKF.packed2unpacked(&p2[1]);
         break;
       case FLDSTA:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.STA.packed2unpacked(&p2[1]);
         break;
       case FLDFIN:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.FIN.packed2unpacked(&p2[1]);
         break;
       case FLDLDG:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.LDG.packed2unpacked(&p2[1]);
         break;
       case FLDTP1:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[0].packed2unpacked(&p2[1]);
         break;
       case FLDTP2:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[1].packed2unpacked(&p2[1]);
         break;
       case FLDTP3:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[2].packed2unpacked(&p2[1]);
         break;
 			  case FLDTP4:
         task.TP[3].packed2unpacked(&p2[1]);
         break;
       case FLDTP5:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[4].packed2unpacked(&p2[1]);
         break;
       case FLDTP6:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[5].packed2unpacked(&p2[1]);
         break;
       case FLDTP7:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[6].packed2unpacked(&p2[1]);
         break;
       case FLDTP8:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[7].packed2unpacked(&p2[1]);
         break;
       case FLDTP9:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[8].packed2unpacked(&p2[1]);
         break;
       case FLDTP10:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[9].packed2unpacked(&p2[1]);
         break;
       case FLDTP11:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[10].packed2unpacked(&p2[1]);
         break;
       case FLDTP12:
+        if (p2 + 1 + 12 > end)
+          return 0;
+
         task.TP[11].packed2unpacked(&p2[1]);
         break;
 
@@ -760,6 +829,9 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
       case FLDPLT2: // Reading pilotname
       case FLDPLT3: // Reading pilotname
       case FLDPLT4: // Reading pilotname
+        if (p2 + 1 + sizeof(PILOT) > end)
+          return 0;
+
         // _fmemcpy(igcheader.PLT, &p2[1], (sizeof igcheader.PLT));
         // igcheader.PLT[(sizeof igcheader.PLT)-1] = 0;
         //_fmemcpy(PILOT, &p2[1], (sizeof PILOT));
@@ -770,21 +842,33 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
           strcat(igcheader.PLT, " ");
         break;
       case FLDGTY: // Reading plane type
+        if (p2 + 1 + sizeof(igcheader.GTY) > end)
+          return 0;
+
         //_fmemcpy(igcheader.GTY, &p2[1], (sizeof igcheader.GTY));
         memcpy(igcheader.GTY, &p2[1], (sizeof igcheader.GTY));
         igcheader.GTY[(sizeof igcheader.GTY) - 1] = 0;
         break;
       case FLDGID: // Reading plane reg
+        if (p2 + 1 + sizeof(igcheader.GID) > end)
+          return 0;
+
         //_fmemcpy(igcheader.GID, &p2[1], (sizeof igcheader.GID));
         memcpy(igcheader.GID, &p2[1], (sizeof igcheader.GID));
         igcheader.GID[(sizeof igcheader.GID) - 1] = 0;
         break;
       case FLDCCL: // Reading plane class
+        if (p2 + 1 + sizeof(igcheader.CCL) > end)
+          return 0;
+
         //_fmemcpy(igcheader.CCL, &p2[1], (sizeof igcheader.CCL));
         memcpy(igcheader.CCL, &p2[1], (sizeof igcheader.CCL));
         igcheader.CCL[(sizeof igcheader.CCL) - 1] = 0;
         break;
       case FLDCID: // Reading plane competition sign
+        if (p2 + 1 + sizeof(igcheader.CID) > end)
+          return 0;
+
         //_fmemcpy(igcheader.CID, &p2[1], (sizeof igcheader.CID));
         memcpy(igcheader.CID, &p2[1], (sizeof igcheader.CID));
         igcheader.CID[(sizeof igcheader.CID) - 1] = 0;
@@ -795,10 +879,11 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
         //dsa_y_b[0] = 2; dsa_y_b[1] = 0;
         //memset(&dsa_y_b[2],0,(sizeof dsa_y_b)-2);
 
-        *serno = (256L * p2[1] + p2[2]);
+        if (p2 + 7 >= end)
+          return 0;
 
         // sonstiges einlesen
-        wordtoserno(igcheader.A, *serno);
+        wordtoserno(igcheader.A, 256 * p2[1] + p2[2]);
 
         sprintf(igcheader.DTM, "%03u", p2[3]);
         sprintf(igcheader.RHW, "%0X.%0X", p2[4] >> 4, (p2[4] & 0xf));
@@ -882,7 +967,7 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
       l = 1;
       break;
     case rectyp_pos:
-    case rectyp_poc:
+    case rectyp_poc: {
       if (p[2] & 0x80) { // Endebedingung
         ende = 1;
         l = 0;
@@ -893,7 +978,8 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
       realtime.tm_isdst = -1;
       mktime(&realtime);
       igcfix.valid = ((p[0] & 0x10) >> 4) ? 'A' : 'V';
-      igcfix.press = ((word)p[0] & 0x0f) << 8 | p[1];
+      const unsigned press = unsigned(p[0] & 0x0f) << 8 | p[1];
+      unsigned gpalt, fxa, enl;
       if (Haupttyp == rectyp_pos) {
         l = pos_ds_size[bfv][0];
         igcfix.lat = ((unsigned long)p[3] & 0x7f) << 16 | ((unsigned long)p[4])
@@ -906,9 +992,9 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
         if (p[9] & 0x80)
           igcfix.lon = -igcfix.lon;
 
-        igcfix.gpalt = ((word)p[9] & 0x70) << 4 | p[10];
-        igcfix.fxa = hdop2fxa(p[9] & 0x0f);
-        igcfix.enl = 4 * p[11];
+        gpalt = unsigned(p[9] & 0x70) << 4 | p[10];
+        fxa = hdop2fxa(p[9] & 0x0f);
+        enl = 4 * p[11];
       }
 
       else {
@@ -922,36 +1008,38 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
 
         igcfix.lat += delta_lat;
         igcfix.lon += delta_lon;
-        igcfix.gpalt = ((word)p[6] & 0x70) << 4 | p[7];
-        igcfix.fxa = hdop2fxa(p[6] & 0x0f);
-        igcfix.enl = 4 * p[8];
+        gpalt = unsigned(p[6] & 0x70) << 4 | p[7];
+        fxa = hdop2fxa(p[6] & 0x0f);
+        enl = 4 * p[8];
       }
-      igcfix.latdeg = labs(igcfix.lat) / 60000;
-      igcfix.latmin = labs(igcfix.lat) % 60000;
-      igcfix.londeg = labs(igcfix.lon) / 60000;
-      igcfix.lonmin = labs(igcfix.lon) % 60000;
 
-      igcfix.gps_alt = 10L * igcfix.gpalt - 1000L;
+      const unsigned latdeg = labs(igcfix.lat) / 60000;
+      const unsigned latmin = labs(igcfix.lat) % 60000;
+      const unsigned londeg = labs(igcfix.lon) / 60000;
+      const unsigned lonmin = labs(igcfix.lon) % 60000;
+
+      long gps_alt = 10L * gpalt - 1000L;
 
       if (igcfile_version >= 423)
-        igcfix.enl = enlflt(igcfix.enl);
-      igcfix.enl = enllim(igcfix.enl);
+        enl = enlflt(enl);
+      enl = enllim(enl);
 
       // Bei allen neuen Dateien auf Wunsch von IAN
       // aber dank neuer Regeln ab
       // Konverter Nr. 4.20 nicht mehr !!
       if ((igcfile_version >= 413) && (igcfile_version < 420))
         if (igcfix.valid == 'V')
-          igcfix.gps_alt = 0;
+          gps_alt = 0;
 
-      igcfix.pressure_alt = pressure2altitude(igcfix.press);
+      const long pressure_alt = pressure2altitude(press);
 
       strftime(igcfix.time,sizeof(igcfix.time),"%H%M%S",&realtime);
       fprintf(Ausgabedatei, "B%6s%02u%05u%c%03u%05u%c%c%05ld%05ld%03u",
-          igcfix.time, igcfix.latdeg, igcfix.latmin, ((igcfix.lat < 0) ? 'S'
-              : 'N'), igcfix.londeg, igcfix.lonmin, ((igcfix.lon < 0) ? 'W'
-              : 'E'), igcfix.valid, igcfix.pressure_alt, igcfix.gps_alt,
-          igcfix.fxa);
+              igcfix.time,
+              latdeg, latmin, igcfix.lat < 0 ? 'S' : 'N',
+              londeg, lonmin, igcfix.lon < 0 ? 'W' : 'E',
+              igcfix.valid, pressure_alt, gps_alt,
+              fxa);
 
       // activate on ENL in I record
       if ((igcfile_version >= 413) && (igcfile_version < 416))
@@ -959,9 +1047,10 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
 
       // have to be active, if sensor exists
       if (strcmp(igcheader.RHW, "3.3") >= 0)
-        fprintf(Ausgabedatei, "%03u", igcfix.enl);
+        fprintf(Ausgabedatei, "%03u", enl);
 
       fprintf(Ausgabedatei, "\n");
+    }
       break;
 
     case rectyp_vrb:
@@ -1012,14 +1101,16 @@ convert_gcs(int igcfile_version, FILE *Ausgabedatei, uint8_t *bin_puffer,
 }
 
 // Members of class DIR
-int
-conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
+bool
+conv_dir(std::vector<DIRENTRY> &flights, const uint8_t *p, const size_t length)
 {
-  int number_of_flights;
+  const uint8_t *const end = p + length;
+
+  assert(flights.empty());
   DIRENTRY de; // directory entry
   uint8_t Haupttyp, Untertyp;
   uint8_t l; // length of DS
-  uint8_t *p2; // Pointer to the beginning of the content of a vrb or vrt
+  const uint8_t *p2; // Pointer to the beginning of the content of a vrb or vrt
   tm olddate;
   memset(&olddate, 0, sizeof(olddate));
 
@@ -1030,13 +1121,15 @@ conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
   memset(&timetm1, 0, sizeof(timetm1));
 
   int bfv = 0;
-  number_of_flights = 0;
   char pilot1[17];
   char pilot2[17];
   char pilot3[17];
   char pilot4[17];
   memset(&de, 0, sizeof(de));
-  while (1) {//number_of_flights < MAXDIRENTRY) {
+
+  size_t nbytes = 0;
+
+  while (p < end) {
     Haupttyp = (p[0] & rectyp_msk);
     switch (Haupttyp) {
     case rectyp_sep: // Initialize Dir-Entry
@@ -1049,11 +1142,17 @@ conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
       de.takeoff = 0;
       bfv = p[0] & ~rectyp_msk;
       if (bfv > max_bfv)
-        return -1;
+      {
+        //abort function
+        return false;
+      }
       l = 1;
       break;
     case rectyp_vrt: // getim'ter variabler DS oder
     case rectyp_vrb: // ungetim'ter variabler DS
+      if (p + 1 >= end)
+        return false;
+
       l = p[1];
       switch (Haupttyp) {
       case rectyp_vrb:
@@ -1066,33 +1165,58 @@ conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
         p2 = p;
         break;
       }
+
+      if (p2 >= end)
+        return false;
+
       Untertyp = (p2[0]);
       switch (Untertyp) {
       case FLDCID: // Read pilotname
+        if (p2 + 1 + sizeof(de.competitionid) > end)
+          return false;
+
         memcpy(de.competitionid, &p2[1], sizeof(de.competitionid));
         de.competitionid[sizeof(de.competitionid) - 1] = 0;
         break;
       case FLDGID: // Read pilotname
+        if (p2 + 1 + sizeof(de.gliderid) > end)
+          return false;
+
         memcpy(de.gliderid, &p2[1], sizeof(de.gliderid));
         de.gliderid[sizeof(de.gliderid) - 1] = 0;
         break;
       case FLDPLT1: // Read pilotname
+        if (p2 + 1 + sizeof(pilot1) > end)
+          return false;
+
         memcpy(pilot1, &p2[1], sizeof(pilot1));
         pilot1[sizeof(pilot1) - 1] = 0;
         break;
       case FLDPLT2: // Read pilotname
+        if (p2 + 1 + sizeof(pilot2) > end)
+          return false;
+
         memcpy(pilot2, &p2[1], sizeof(pilot2));
         pilot2[sizeof(pilot2) - 1] = 0;
         break;
       case FLDPLT3: // Read pilotname
+        if (p2 + 1 + sizeof(pilot3) > end)
+          return false;
+
         memcpy(pilot3, &p2[1], sizeof(pilot3));
         pilot3[sizeof(pilot3) - 1] = 0;
         break;
       case FLDPLT4: // Read pilotname
+        if (p2 + 1 + sizeof(pilot4) > end)
+          return false;
+
         memcpy(pilot4, &p2[1], sizeof(pilot4));
         pilot4[sizeof(pilot4) - 1] = 0;
         break;
       case FLDHDR: // Read serial number
+        if (p2 + 2 >= end)
+          return false;
+
         de.serno = 256 * p2[1] + p2[2];
         break;
       case FLDETKF: // set takeoff flag
@@ -1107,13 +1231,30 @@ conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
     case rectyp_pos:
       l = pos_ds_size[bfv][0];
       break;
+
+
+    /*
+     * This End Condition Statement does not seem to be
+     * valid. At least not for the tested Volkslogger.
+     * To have a valid condition the length is
+     * now known in conv_dir() and the loop will finish when the
+     * end of the data is reached.
+     * Maybe this end condition here is valid in special
+     * cases or on other hardware though.
+     */
     case rectyp_poc:
+      if (p + 2 >= end)
+        return false;
+
       if (p[2] & 0x80) { // Endebedingung
-        return number_of_flights;
+        return true;
       }
       l = pos_ds_size[bfv][1];
       break;
     case rectyp_tnd:
+      if (p + 7 >= end)
+        return false;
+
       // speichert in timetm1 den aktuellen tnd-DS ab
       temptime = 65536L * p[2] + 256L * p[3] + p[4];
       timetm1.tm_sec = temptime % 3600;
@@ -1130,52 +1271,54 @@ conv_dir(DIRENTRY* flights, uint8_t *p, int countonly)
       l = 8;
       break;
     case rectyp_end:
-      if (!countonly) {
-        // setzt firsttime und lasttime aufgrund der Werte im sta-DS
-        temptime = 65536L * p[4] + 256L * p[5] + p[6]; // Aufzeichnungsbeginn
-        de.firsttime = timetm1;
-        de.firsttime.tm_sec -= temptime % 3600;
+      if (p + 6 >= end)
+        return false;
 
-        de.firsttime.tm_hour -= temptime / 3600;
-        de.firsttime.tm_isdst = -1;
-        mktime(&de.firsttime);
-        de.lasttime = de.firsttime;
+      // setzt firsttime und lasttime aufgrund der Werte im sta-DS
+      temptime = 65536L * p[4] + 256L * p[5] + p[6]; // Aufzeichnungsbeginn
+      de.firsttime = timetm1;
+      de.firsttime.tm_sec -= temptime % 3600;
 
-        temptime = 65536L * p[1] + 256L * p[2] + p[3]; // Aufzeichnungsdauer
-        de.recordingtime = temptime;
-        de.lasttime.tm_sec += temptime % 3600;
-        de.lasttime.tm_hour += temptime / 3600;
-        de.lasttime.tm_isdst = -1;
-        mktime(&de.lasttime);
+      de.firsttime.tm_hour -= temptime / 3600;
+      de.firsttime.tm_isdst = -1;
+      mktime(&de.firsttime);
+      de.lasttime = de.firsttime;
 
-        if (!olddate_flg) {
-          olddate = de.firsttime;
-          flight_of_day = 0;
-          olddate_flg = 1;
-        }
-        if ((olddate.tm_mday == de.firsttime.tm_mday) && (olddate.tm_mon
-            == de.firsttime.tm_mon)
-            && (olddate.tm_year == de.firsttime.tm_year))
-          flight_of_day++;
-        else {
-          olddate = de.firsttime;
-          flight_of_day = 1;
-          olddate_flg = 1;
-        }
-        strcat(de.pilot, pilot1);
-        strcat(de.pilot, pilot2);
-        strcat(de.pilot, pilot3);
-        strcat(de.pilot, pilot4);
+      temptime = 65536L * p[1] + 256L * p[2] + p[3]; // Aufzeichnungsdauer
+      de.recordingtime = temptime;
+      de.lasttime.tm_sec += temptime % 3600;
+      de.lasttime.tm_hour += temptime / 3600;
+      de.lasttime.tm_isdst = -1;
+      mktime(&de.lasttime);
 
-        flights[number_of_flights] = de;
+      if (!olddate_flg) {
+        olddate = de.firsttime;
+        flight_of_day = 0;
+        olddate_flg = 1;
       }
-      number_of_flights++;
+      if ((olddate.tm_mday == de.firsttime.tm_mday) && (olddate.tm_mon
+          == de.firsttime.tm_mon)
+          && (olddate.tm_year == de.firsttime.tm_year))
+        flight_of_day++;
+      else {
+        olddate = de.firsttime;
+        flight_of_day = 1;
+        olddate_flg = 1;
+      }
+      strcat(de.pilot, pilot1);
+      strcat(de.pilot, pilot2);
+      strcat(de.pilot, pilot3);
+      strcat(de.pilot, pilot4);
+
+      flights.push_back(de);
       l = 7;
       break;
     default:
-      return -1;
+      //abort function
+      return false;
     };
     p += l;
+    nbytes += l;
   }
-  return -1;
+  return true;
 }
