@@ -49,7 +49,7 @@ doc/html/advanced/input/ALL		http://xcsoar.sourceforge.net/advanced/input/
 #include "LogFile.hpp"
 #include "Device/Parser.hpp"
 #include "UIState.hpp"
-#include "ComputerSettings.hpp"
+#include "Computer/Settings.hpp"
 #include "MapSettings.hpp"
 #include "Math/FastMath.h"
 #include "Dialogs/Dialogs.h"
@@ -59,7 +59,7 @@ doc/html/advanced/input/ALL		http://xcsoar.sourceforge.net/advanced/input/
 #include "Dialogs/Task/TaskDialogs.hpp"
 #include "Dialogs/Traffic/TrafficDialogs.hpp"
 #include "Dialogs/Waypoint/WaypointDialogs.hpp"
-#include "Dialogs/Weather.hpp"
+#include "Dialogs/Weather/WeatherDialogs.hpp"
 #include "Dialogs/Plane/PlaneDialogs.hpp"
 #include "Dialogs/Message.hpp"
 #include "Dialogs/dlgAnalysis.hpp"
@@ -270,10 +270,10 @@ InputEvents::eventWaypointDetails(const TCHAR *misc)
       return;
     }
   } else if (StringIsEqual(misc, _T("select"))) {
-    wp = ShowWaypointListDialog(*CommonInterface::main_window, basic.location);
+    wp = ShowWaypointListDialog(basic.location);
   }
   if (wp)
-    dlgWaypointDetailsShowModal(*CommonInterface::main_window, *wp);
+    dlgWaypointDetailsShowModal(*wp);
 }
 
 // StatusMessage
@@ -477,7 +477,7 @@ InputEvents::eventSetup(const TCHAR *misc)
   else if (StringIsEqual(misc, _T("Plane")))
     dlgPlanesShowModal();
   else if (StringIsEqual(misc, _T("Alternates")))
-    dlgAlternatesListShowModal(*CommonInterface::main_window);
+    dlgAlternatesListShowModal();
 
   trigger_redraw();
 }
