@@ -30,6 +30,7 @@ Copyright_License {
 #include "Screen/Layout.hpp"
 #include "Logger/Logger.hpp"
 #include "Task/ProtectedTaskManager.hpp"
+#include "Engine/Task/Ordered/OrderedTask.hpp"
 #include "Renderer/TextInBox.hpp"
 #include "Screen/UnitSymbol.hpp"
 #include "Terrain/RasterWeather.hpp"
@@ -104,6 +105,11 @@ GlueMapWindow::DrawPanInfo(Canvas &canvas) const
   PixelScalar y = 0 + padding;
   PixelScalar x = render_projection.GetScreenWidth() - padding;
 
+  if (compass_visible)
+    /* don't obscure the north arrow */
+    /* TODO: obtain offset from CompassRenderer */
+    y += Layout::Scale(19) + Layout::FastScale(13);
+
   if (terrain) {
     short elevation = terrain->GetTerrainHeight(location);
     if (!RasterBuffer::IsSpecial(elevation)) {
@@ -237,11 +243,41 @@ GlueMapWindow::DrawFlightMode(Canvas &canvas, const PixelRect &rc) const
 void
 GlueMapWindow::DrawFinalGlide(Canvas &canvas, const PixelRect &rc) const
 {
+
+  if (GetMapSettings().final_glide_bar_display_mode==FinalGlideBarDisplayMode::OFF)
+    return;
+
+  if (GetMapSettings().final_glide_bar_display_mode==FinalGlideBarDisplayMode::AUTO) {
+    const TaskStats &task_stats = Calculated().task_stats;
+    const ElementStat &total = task_stats.total;
+    const GlideResult &solution = total.solution_remaining;
+    const GlideResult &solution_mc0 = total.solution_mc0;
+    const GlideSettings &glide_settings= GetComputerSettings().task.glide;
+
+    if (!task_stats.task_valid || !solution.IsOk() || !solution_mc0.IsDefined())
+      return;
+
+    if (solution_mc0.SelectAltitudeDifference(glide_settings) < fixed(-1000) &&
+        solution.SelectAltitudeDifference(glide_settings) < fixed(-1000))
+      return;
+  }
+
   final_glide_bar_renderer.Draw(canvas, rc, Calculated(),
                                 GetComputerSettings().task.glide,
                                 GetMapSettings().final_glide_bar_mc0_enabled);
 }
 
+void
+GlueMapWindow::DrawVario(Canvas &canvas, const PixelRect &rc) const
+{
+  if (!GetMapSettings().vario_bar_enabled)
+   return;
+
+  vario_bar_renderer.Draw(canvas, rc, Basic(), Calculated(),
+                                GetComputerSettings().polar.glide_polar_task,
+                                true); //NOTE: AVG enabled for now, make it configurable ;
+}
+
 void
 GlueMapWindow::DrawMapScale(Canvas &canvas, const PixelRect &rc,
                             const MapWindowProjection &projection) const
@@ -378,7 +414,7 @@ GlueMapWindow::DrawThermalBand(Canvas &canvas, const PixelRect &rc) const
   tb_rect.left = rc.left;
   tb_rect.right = rc.left+Layout::Scale(20);
   tb_rect.top = Layout::Scale(2);
-  tb_rect.bottom = (rc.bottom-rc.top)/2-Layout::Scale(73);
+  tb_rect.bottom = (rc.bottom-rc.top)/5 - Layout::Scale(2);
 
   const ThermalBandRenderer &renderer = thermal_band_renderer;
   if (task != NULL) {
@@ -390,7 +426,7 @@ GlueMapWindow::DrawThermalBand(Canvas &canvas, const PixelRect &rc) const
                              tb_rect,
                              GetComputerSettings().task,
                              true,
-                             &task_manager->GetOrderedTaskBehaviour());
+                             &task_manager->GetOrderedTask().GetOrderedTaskBehaviour());
   } else {
     renderer.DrawThermalBand(Basic(),
                              Calculated(),
