@@ -49,6 +49,7 @@ Copyright_License {
 #include "Units/Descriptor.hpp"
 #include "LocalPath.hpp"
 #include "Math/Angle.hpp"
+#include "Util/ConvertString.hpp"
 
 #include <windef.h>
 #include <assert.h>
@@ -418,10 +419,11 @@ RowFormWidget::AddTime(const TCHAR *label, const TCHAR *help,
 
 WndProperty *
 RowFormWidget::AddRoughTime(const TCHAR *label, const TCHAR *help,
-                            RoughTime value, DataFieldListener *listener)
+                            RoughTime value, RoughTimeDelta time_zone,
+                            DataFieldListener *listener)
 {
   WndProperty *edit = Add(label, help);
-  RoughTimeDataField *df = new RoughTimeDataField(value, listener);
+  RoughTimeDataField *df = new RoughTimeDataField(value, time_zone, listener);
   edit->SetDataField(df);
   return edit;
 }
@@ -440,7 +442,7 @@ RowFormWidget::AddSpacer()
 
 WndProperty *
 RowFormWidget::AddFileReader(const TCHAR *label, const TCHAR *help,
-                             const TCHAR *registry_key, const TCHAR *filters,
+                             const char *registry_key, const TCHAR *filters,
                              bool nullable)
 {
   WndProperty *edit = Add(label, help);
@@ -450,15 +452,13 @@ RowFormWidget::AddFileReader(const TCHAR *label, const TCHAR *help,
   if (nullable)
     df->AddNull();
 
-  size_t length;
-  while ((length = _tcslen(filters)) > 0) {
-    df->ScanDirectoryTop(filters);
-    filters += length + 1;
-  }
+  df->ScanMultiplePatterns(filters);
 
-  TCHAR path[MAX_PATH];
-  if (Profile::GetPath(registry_key, path))
-    df->Lookup(path);
+  if (registry_key != nullptr) {
+    TCHAR path[MAX_PATH];
+    if (Profile::GetPath(registry_key, path))
+      df->Lookup(path);
+  }
 
   edit->RefreshDisplay();
 
@@ -747,7 +747,7 @@ RowFormWidget::SaveValue(unsigned i, TCHAR *string, size_t max_size) const
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          TCHAR *string, size_t max_size) const
 {
   if (!SaveValue(i, string, max_size))
@@ -758,7 +758,7 @@ RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          bool &value, bool negated) const
 {
   if (!SaveValue(i, value, negated))
@@ -769,7 +769,7 @@ RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          int &value) const
 {
   if (!SaveValue(i, value))
@@ -780,7 +780,7 @@ RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          uint8_t &value) const
 {
   if (!SaveValue(i, value))
@@ -791,7 +791,7 @@ RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          uint16_t &value) const
 {
   if (!SaveValue(i, value))
@@ -802,7 +802,7 @@ RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
 }
 
 bool
-RowFormWidget::SaveValue(unsigned i, const TCHAR *registry_key,
+RowFormWidget::SaveValue(unsigned i, const char *registry_key,
                          fixed &value) const
 {
   if (!SaveValue(i, value))
@@ -819,60 +819,60 @@ RowFormWidget::SaveValue(unsigned i, UnitGroup unit_group, fixed &value) const
     (const DataFieldFloat &)GetDataField(i);
   assert(df.GetType() == DataField::Type::REAL);
 
-  fixed new_value = df.GetAsFixed();
   const Unit unit = Units::GetUserUnitByGroup(unit_group);
-  new_value = Units::ToSysUnit(new_value, unit);
-  if (new_value > value - df.GetStep() / 2 &&
-      new_value < value + df.GetStep() / 2)
+  fixed new_value = df.GetAsFixed();
+  fixed old_value = Units::ToUserUnit(value, unit);
+
+  if (fabs(new_value - old_value) < df.GetStep() / 100)
     return false;
 
-  value = new_value;
+  value = Units::ToSysUnit(new_value, unit);
   return true;
 }
 
 bool
 RowFormWidget::SaveValue(unsigned i, UnitGroup unit_group,
-                         const TCHAR *registry_key, fixed &value) const
+                         const char *registry_key, fixed &value) const
 {
   const DataFieldFloat &df =
     (const DataFieldFloat &)GetDataField(i);
   assert(df.GetType() == DataField::Type::REAL);
 
-  fixed new_value = df.GetAsFixed();
   const Unit unit = Units::GetUserUnitByGroup(unit_group);
-  new_value = Units::ToSysUnit(new_value, unit);
-  if (new_value > value - df.GetStep() / 2 &&
-      new_value < value + df.GetStep() / 2)
+  fixed new_value = df.GetAsFixed();
+  fixed old_value = Units::ToUserUnit(value, unit);
+
+  if (fabs(new_value - old_value) < df.GetStep() / 100)
     return false;
 
-  value = new_value;
-  Profile::Set(registry_key, new_value);
+  value = Units::ToSysUnit(new_value, unit);
+  Profile::Set(registry_key, value);
   return true;
 }
 
 bool
 RowFormWidget::SaveValue(unsigned i, UnitGroup unit_group,
-                         const TCHAR *registry_key, unsigned int &value) const
+                         const char *registry_key, unsigned int &value) const
 {
   const DataFieldFloat &df =
     (const DataFieldFloat &)GetDataField(i);
   assert(df.GetType() == DataField::Type::INTEGER ||
          df.GetType() == DataField::Type::REAL);
 
-  fixed new_value = df.GetAsFixed();
   const Unit unit = Units::GetUserUnitByGroup(unit_group);
-  new_value = Units::ToSysUnit(new_value, unit);
-  if ((unsigned int)(new_value) > value - iround(df.GetStep()) / 2 &&
-      (unsigned int)(new_value) < value + iround(df.GetStep()) / 2)
+  fixed new_value = df.GetAsFixed();
+  fixed old_value = Units::ToUserUnit(fixed(value), unit);
+
+  if (fabs(new_value - old_value) < df.GetStep() / 100)
     return false;
 
-  value = iround(new_value);
+  value = iround(Units::ToSysUnit(new_value, unit));
   Profile::Set(registry_key, value);
   return true;
 }
 
 bool
-RowFormWidget::SaveValueFileReader(unsigned i, const TCHAR *registry_key)
+RowFormWidget::SaveValueFileReader(unsigned i, const char *registry_key)
 {
   const DataFieldFileReader *dfe =
     (const DataFieldFileReader *)GetControl(i).GetDataField();
@@ -880,11 +880,15 @@ RowFormWidget::SaveValueFileReader(unsigned i, const TCHAR *registry_key)
   _tcscpy(new_value, dfe->GetPathFile());
   ContractLocalPath(new_value);
 
-  const TCHAR *old_value = Profile::Get(registry_key, _T(""));
-  if (_tcscmp(old_value, new_value) == 0)
+  const WideToUTF8Converter new_value2(new_value);
+  if (!new_value2.IsValid())
     return false;
 
-  Profile::Set(registry_key, new_value);
+  const char *old_value = Profile::Get(registry_key, "");
+  if (StringIsEqual(old_value, new_value2))
+    return false;
+
+  Profile::Set(registry_key, new_value2);
   return true;
 }
 
